//------------------------------------------------------------------------------
// The Delphi Container Library
// Jean-Philippe BEMPEL aka RDM
// rdm_30@yahoo.com
//------------------------------------------------------------------------------
unit HashMap;

{$I dcl.inc}

interface

uses DCL_intf, AbstractContainer;

type
  IIntfIntfEntry = interface
  ['{E9D0A4E5-3E1F-49F0-8B1B-9662908E8692}']
    function Equals(AObject: IIntfIntfEntry): Boolean;
    function GetKey: IInterface;
    function GetValue: IInterface;
    procedure SetValue(AObject: IInterface);
    property Key: IInterface read GetKey;
    property Value: IInterface read GetValue write SetValue;
  end;

  TIntfIntfEntry = class(TInterfacedObject, IIntfIntfEntry)
  private
    FKey: IInterface;
    FValue: IInterface;
  protected
    function Equals(AObject: IIntfIntfEntry): Boolean;
    function GetKey: IInterface;
    function GetValue: IInterface;
    procedure SetValue(AObject: IInterface);
  public
    constructor Create(Key, Value: IInterface);
    destructor Destroy; override;
  end;

	IStrIntfEntry = interface
  ['{E8E58DA2-151C-499C-BCF0-CFFA95AD030B}']
    function Equals(AObject: IStrIntfEntry): Boolean;
    function GetKey: string;
    function GetValue: IInterface;
    procedure SetValue(AObject: IInterface);
    property Key: string read GetKey;
    property Value: IInterface read GetValue write SetValue;
  end;

  TStrIntfEntry = class(TAbstractContainer, IStrIntfEntry)
  private
    FKey: string;
    FValue: IInterface;
  protected
    function Equals(AObject: IStrIntfEntry): Boolean;
    function GetKey: string;
    function GetValue: IInterface;
    procedure SetValue(AObject: IInterface);
  public
    constructor Create(Key: string; Value: IInterface);
    destructor Destroy; override;
  end;

	IStrStrEntry = interface
  ['{41B01FF4-2D28-429C-B9D7-E0230F7E4D0B}']
    function Equals(AObject: IStrStrEntry): Boolean;
    function GetKey: string;
    function GetValue: string;
    procedure SetValue(AObject: string);
    property Key: string read GetKey;
    property Value: string read GetValue write SetValue;
  end;

  TStrStrEntry = class(TInterfacedObject, IStrStrEntry)
  private
    FKey: string;
    FValue: string;
  protected
    function Equals(AObject: IStrStrEntry): Boolean;
    function GetKey: string;
    function GetValue: string;
    procedure SetValue(AObject: string);
  public
    constructor Create(Key, Value: string);
    destructor Destroy; override;
  end;

  TStrEntry = class(TObject)
  private
    FKey: string;
    FValue: TObject;
    FOwnsObjects: Boolean;
  public
    function Equals(AObject: TObject): Boolean;
  public
    constructor Create(const Key: string; Value: TObject; AOwnsObjects: Boolean);
    destructor Destroy; override;
    property Key: string read FKey;
    property Value: TObject read FValue write FValue;
  end;

  TEntry = class(TObject)
  private
    FKey: TObject;
    FValue: TObject;
    FOwnsObjects: Boolean;
  public
    function Equals(AObject: TObject): Boolean;
  public
    constructor Create(Key, Value: TObject; AOwnsObjects: Boolean);
    destructor Destroy; override;
    property Key: TObject read FKey;
    property Value: TObject read FValue write FValue;
  end;

  TIntfIntfEntryArray = array of IIntfIntfEntry;
  TStrIntfEntryArray = array of IStrIntfEntry;
  TStrStrEntryArray = array of IStrStrEntry;
  TStrEntryArray = array of TStrEntry;
  TEntryArray = array of TEntry;

  PIntfIntfBucket = ^TIntfIntfBucket;
  TIntfIntfBucket = record
    Count: Integer;
    Entries: TIntfIntfEntryArray;
  end;

  PStrIntfBucket = ^TStrIntfBucket;
  TStrIntfBucket = record
    Count: Integer;
    Entries: TStrIntfEntryArray;
  end;

  PStrStrBucket = ^TStrStrBucket;
  TStrStrBucket = record
    Count: Integer;
    Entries: TStrStrEntryArray;
  end;

  PStrBucket = ^TStrBucket;
  TStrBucket = record
    Count: Integer;
    Entries: TStrEntryArray;
  end;

  PBucket = ^TBucket;
  TBucket = record
    Count: Integer;
    Entries: TEntryArray;
  end;

  TIntfIntfBucketArray = array of TIntfIntfBucket;
  TStrIntfBucketArray = array of TStrIntfBucket;
  TStrStrBucketArray = array of TStrStrBucket;
  TStrBucketArray = array of TStrBucket;
  TBucketArray = array of TBucket;

  // Hash Function
  THashFunction = function (Key: Integer): Integer of object;

	TIntfIntfHashMap = class(TAbstractContainer, IIntfIntfMap, IIntfCloneable)
  private
    FCapacity: Integer;
    FCount: Integer;
    FBuckets: TIntfIntfBucketArray;
    FHashFunction: THashFunction;
    function HashMul(Key: Integer): Integer;
  protected
    procedure GrowEntries(BucketIndex: Integer); virtual;
  protected
  { IIntfIntfMap }
    procedure Clear;
    function ContainsKey(Key: IInterface): Boolean;
    function ContainsValue(Value: IInterface): Boolean;
    function EntrySet: IIntfSet;
    function Equals(AMap: IIntfIntfMap): Boolean;
    function GetValue(Key: IInterface): IInterface;
    function IsEmpty: Boolean;
    function KeySet: IIntfSet;
    procedure PutAll(AMap: IIntfIntfMap);
    procedure PutValue(Key, Value: IInterface);
    function Remove(Key: IInterface): IInterface;
  	function Size: Integer;
    function Values: IIntfCollection;
  protected
  { IIntfCloneable }
  	function Clone: IInterface;
  public
    constructor Create; overload;
    constructor Create(Capacity: Integer); overload;
    destructor Destroy; override;
    property HashFunction: THashFunction read FHashFunction write FHashFunction;
  end;

	TStrIntfHashMap = class(TAbstractContainer, IStrIntfMap, IIntfCloneable)
  private
    FCapacity: Integer;
    FCount: Integer;
    FBuckets: TStrIntfBucketArray;
    FHashFunction: THashFunction;
    function HashMul(Key: Integer): Integer;
    function HashString(const Key: string): Integer;
  protected
    procedure GrowEntries(BucketIndex: Integer); virtual;
  protected
  { IIntfMap }
    procedure Clear;
    function ContainsKey(const Key: string): Boolean;
    function ContainsValue(Value: IInterface): Boolean;
    function EntrySet: IIntfSet;
    function Equals(AMap: IStrIntfMap): Boolean;
    function GetValue(const Key: string): IInterface;
    function IsEmpty: Boolean;
    function KeySet: IStrSet;
    procedure PutAll(AMap: IStrIntfMap);
    procedure PutValue(const Key: string; Value: IInterface);
    function Remove(const Key: string): IInterface;
  	function Size: Integer;
    function Values: IIntfCollection;
  protected
  { IIntfCloneable }
  	function Clone: IInterface;
  public
    constructor Create; overload;
    constructor Create(Capacity: Integer); overload;
    destructor Destroy; override;
    property HashFunction: THashFunction read FHashFunction write FHashFunction;
  end;

	TStrStrHashMap = class(TAbstractContainer, IStrStrMap, IIntfCloneable)
  private
    FCapacity: Integer;
    FCount: Integer;
    FBuckets: TStrStrBucketArray;
    FHashFunction: THashFunction;
    function HashMul(Key: Integer): Integer;
    function HashString(const Key: string): Integer;
  protected
    procedure GrowEntries(BucketIndex: Integer); virtual;
  protected
  { IStrStrMap }
    procedure Clear;
    function ContainsKey(const Key: string): Boolean;
    function ContainsValue(const Value: string): Boolean;
    function EntrySet: IIntfSet;
    function Equals(AMap: IStrStrMap): Boolean;
    function GetValue(const Key: string): string;
    function IsEmpty: Boolean;
    function KeySet: IStrSet;
    procedure PutAll(AMap: IStrStrMap);
    procedure PutValue(const Key, Value: string);
    function Remove(const Key: string): string;
  	function Size: Integer;
    function Values: IStrCollection;
  protected
  { IIntfCloneable }
  	function Clone: IInterface;
  public
    constructor Create; overload;
    constructor Create(Capacity: Integer); overload;
    destructor Destroy; override;
    property HashFunction: THashFunction read FHashFunction write FHashFunction;
  end;

	TStrHashMap = class(TAbstractContainer, IStrMap, ICloneable)
  private
    FCapacity: Integer;
    FCount: Integer;
    FBuckets: TStrBucketArray;
    FHashFunction: THashFunction;
    FOwnsObjects: Boolean;
    function HashMul(Key: Integer): Integer;
    function HashString(const Key: string): Integer;
  protected
    procedure GrowEntries(BucketIndex: Integer); virtual;
  protected
  { IStrMap }
    procedure Clear;
    function ContainsKey(const Key: string): Boolean;
    function ContainsValue(Value: TObject): Boolean;
    function EntrySet: ISet;
    function Equals(AMap: IStrMap): Boolean;
    function GetValue(const Key: string): TObject;
    function IsEmpty: Boolean;
    function KeySet: IStrSet;
    procedure PutAll(AMap: IStrMap);
    procedure PutValue(const Key: string; Value: TObject);
    function Remove(const Key: string): TObject;
  	function Size: Integer;
    function Values: ICollection;
  protected
  { ICloneable }
  	function Clone: TObject;
  public
    constructor Create; overload;
    constructor Create(Capacity: Integer; AOwnsObjects: Boolean); overload;
    destructor Destroy; override;
    property HashFunction: THashFunction read FHashFunction write FHashFunction;
  end;

	THashMap = class(TAbstractContainer, IMap, ICloneable)
  private
    FCapacity: Integer;
    FCount: Integer;
    FBuckets: TBucketArray;
    FHashFunction: THashFunction;
    FOwnsObjects: Boolean;
    function HashMul(Key: Integer): Integer;
  protected
    procedure GrowEntries(BucketIndex: Integer); virtual;
  public
  { IMap }
    procedure Clear;
    function ContainsKey(Key: TObject): Boolean; 
    function ContainsValue(Value: TObject): Boolean;
    function EntrySet: ISet;
    function Equals(AMap: IMap): Boolean;
    function GetValue(Key: TObject): TObject;
    function IsEmpty: Boolean;
    function KeySet: ISet;
    procedure PutAll(AMap: IMap);
    procedure PutValue(Key, Value: TObject);
    function Remove(Key: TObject): TObject;
  	function Size: Integer;
    function Values: ICollection;
  protected
  { ICloneable }
  	function Clone: TObject;
  public
    constructor Create; overload;
    constructor Create(Capacity: Integer; AOwnsObjects: Boolean); overload;
    destructor Destroy; override;
    property HashFunction: THashFunction read FHashFunction write FHashFunction;
  end;

implementation

uses SysUtils, ArraySet, ArrayList;

{ TIntfIntfEntry }

constructor TIntfIntfEntry.Create(Key, Value: IInterface);
begin
  FKey := Key;
  FValue := Value;
end;

destructor TIntfIntfEntry.Destroy;
begin
  FKey := nil;
  FValue := nil;
  inherited;
end;

function TIntfIntfEntry.Equals(AObject: IIntfIntfEntry): Boolean;
begin
  Result := False;
end;

function TIntfIntfEntry.GetKey: IInterface;
begin
  Result := FKey;
end;

function TIntfIntfEntry.GetValue: IInterface;
begin
  Result := FValue;
end;

procedure TIntfIntfEntry.SetValue(AObject: IInterface);
begin
  FValue := AObject;
end;

{ TStrIntfEntry }

constructor TStrIntfEntry.Create(Key: string; Value: IInterface);
begin
  FKey := Key;
  FValue := Value;
end;

destructor TStrIntfEntry.Destroy;
begin
  FKey := '';
  FValue := nil;
  inherited;
end;

function TStrIntfEntry.Equals(AObject: IStrIntfEntry): Boolean;
begin
  Result := False;
  if AObject.GetKey = FKey then
    if AObject.GetValue = FValue then
      Result := True;
end;

function TStrIntfEntry.GetKey: string;
begin
  Result := FKey;
end;

function TStrIntfEntry.GetValue: IInterface;
begin
  Result := FValue;
end;

procedure TStrIntfEntry.SetValue(AObject: IInterface);
begin
  FValue := AObject;
end;

{ TStrStrEntry }

constructor TStrStrEntry.Create(Key, Value: string);
begin
  FKey := Key;
  FValue := Value;
end;

destructor TStrStrEntry.Destroy;
begin
  FKey := '';
  FValue := '';
  inherited;
end;

function TStrStrEntry.Equals(AObject: IStrStrEntry): Boolean;
begin
  Result := False;
end;

function TStrStrEntry.GetKey: string;
begin
  Result := FKey;
end;

function TStrStrEntry.GetValue: string;
begin
  Result := FValue;
end;

procedure TStrStrEntry.SetValue(AObject: string);
begin
  FValue := AObject;
end;

{ TStrEntry }

constructor TStrEntry.Create(const Key: string; Value: TObject;
  AOwnsObjects: Boolean);
begin
  FKey := Key;
  FValue := Value;
  FOwnsObjects := AOwnsObjects;
end;

destructor TStrEntry.Destroy;
begin
  FKey := '';
  if FOwnsObjects then
    FValue.Free;
  inherited;
end;

function TStrEntry.Equals(AObject: TObject): Boolean;
begin
  Result := False;
end;


{ TEntry }

constructor TEntry.Create(Key, Value: TObject; AOwnsObjects: Boolean);
begin
  FKey := Key;
  FValue := Value;
  FOwnsObjects := AOwnsObjects;
end;

destructor TEntry.Destroy;
begin
  if FOwnsObjects then
  begin
    FValue.Free;
    FKey.Free;
  end;
  inherited;
end;

function TEntry.Equals(AObject: TObject): Boolean;
begin
  Result := False;
end;

{ TIntfIntfHashMap }

procedure TIntfIntfHashMap.Clear;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
      FBuckets[I].Entries[J] := nil;
    FBuckets[I].Count := 0;
  end;
  FCount := 0;
end;

function TIntfIntfHashMap.Clone: IInterface;
var
  I, J: Integer;
  NewEntryArray: TIntfIntfEntryArray;
  NewMap: TIntfIntfHashMap;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  NewMap := TIntfIntfHashMap.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
  begin
    NewEntryArray := NewMap.FBuckets[I].Entries;
    SetLength(NewEntryArray, Length(FBuckets[I].Entries));
    for J := 0 to FBuckets[I].Count - 1 do
      NewEntryArray[J] := TIntfIntfEntry.Create(FBuckets[I].Entries[J].GetKey,
      	FBuckets[I].Entries[J].GetValue);
    NewMap.FBuckets[I].Count := FBuckets[I].Count;
  end;
  Result := NewMap;
end;

function TIntfIntfHashMap.ContainsKey(Key: IInterface): Boolean;
var
  I: Integer;
  Bucket: PIntfIntfBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Key = nil then
    Exit;
  Bucket := @(FBuckets[FHashFunction(Integer(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := True;
      Exit;
    end;
end;

function TIntfIntfHashMap.ContainsValue(Value: IInterface): Boolean;
var
  I, J: Integer;
  Bucket: PIntfIntfBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Value = nil then
    Exit;
  for J := 0 to FCapacity - 1 do
  begin
    Bucket := @(FBuckets[J]);
    for I := 0 to Bucket.Count - 1 do
      if Bucket.Entries[I].GetValue = Value then
      begin
        Result := True;
        Exit;
      end;
  end;
end;

constructor TIntfIntfHashMap.Create(Capacity: Integer);
var
  I: Integer;
begin
  inherited Create;
  FCapacity := Capacity;
  SetLength(FBuckets, FCapacity);
  for I := 0 to FCapacity - 1 do
    SetLength(FBuckets[I].Entries, 1);
  FHashFunction := HashMul;
end;

constructor TIntfIntfHashMap.Create;
begin
  Create(16);
end;

destructor TIntfIntfHashMap.Destroy;
begin
  Clear;
  inherited;
end;

function TIntfIntfHashMap.EntrySet: IIntfSet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TIntfArraySet.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J])
end;

function TIntfIntfHashMap.Equals(AMap: IIntfIntfMap): Boolean;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if AMap = nil then
    Exit;
  if FCount <> AMap.Size then
    Exit;
  Result := True;
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
      if AMap.ContainsKey(FBuckets[I].Entries[J].GetKey) then
      begin
        if not (AMap.GetValue(FBuckets[I].Entries[J].GetKey) = FBuckets[I].Entries[J].GetValue) then
        begin
          Result := False;
          Exit;
        end;
      end
      else
      begin
        Result := False;
        Exit;
      end;
  end;
end;

function TIntfIntfHashMap.GetValue(Key: IInterface): IInterface;
var
  I: Integer;
  Bucket: PIntfIntfBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := nil;
  if Key = nil then
    Exit;
  Bucket := @(FBuckets[FHashFunction(Integer(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := Bucket.Entries[I].GetValue;
      Exit;
    end;
end;

procedure TIntfIntfHashMap.GrowEntries(BucketIndex: Integer);
var
  Capacity: Integer;
begin
  Capacity := Length(FBuckets[BucketIndex].Entries);
  if Capacity > 64 then
	  Capacity := Capacity + Capacity div 4
  else
    Capacity := Capacity * 4;
  SetLength(FBuckets[BucketIndex].Entries, Capacity);
end;

function TIntfIntfHashMap.HashMul(Key: Integer): Integer;
const
  A = 0.6180339887; // (sqrt(5) - 1) / 2
begin
  Result := Trunc(FCapacity * (Frac(Key * A)));
end;

function TIntfIntfHashMap.IsEmpty: Boolean;
begin
  Result := FCount = 0;
end;

function TIntfIntfHashMap.KeySet: IIntfSet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TIntfArraySet.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
	  	Result.Add(FBuckets[I].Entries[J].GetKey)
end;

procedure TIntfIntfHashMap.PutAll(AMap: IIntfIntfMap);
var
  It: IIntfIterator;
  Entry: IIntfIntfEntry;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if AMap = nil then
    Exit;
  It := AMap.EntrySet.First;
  while It.HasNext do
  begin
    Entry := IIntfIntfEntry(It.Next);
    PutValue(Entry.GetKey, Entry.GetValue);
  end;
end;

procedure TIntfIntfHashMap.PutValue(Key, Value: IInterface);
var
  Index: Integer;
  Bucket: PIntfIntfBucket;
  Temp: IIntfIntfEntry;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if Key = nil then
    Exit;
  if Value = nil then
    Exit;
  Index := FHashFunction(Integer(Key));
  Bucket := @(FBuckets[Index]);
  for I := 0 to Bucket.Count - 1 do
  	if Bucket.Entries[I].Key = Key then
    begin
			Bucket.Entries[I].Value := Value;
      Exit;
    end;
  if Bucket.Count = Length(Bucket.Entries) then
    GrowEntries(Index);
  Temp := TIntfIntfEntry.Create(Key, Value);
  Bucket.Entries[Bucket.Count] := Temp;
  Inc(Bucket.Count);
  Inc(FCount);
end;

function TIntfIntfHashMap.Remove(Key: IInterface): IInterface;
var
  Bucket: PIntfIntfBucket;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := nil;
  if Key = nil then
    Exit;
  Bucket := @(FBuckets[FHashFunction(Integer(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := Bucket.Entries[I].Value;
      System.Move(Bucket.Entries[I + 1], Bucket.Entries[I],
      	(Bucket.Count - I) * SizeOf(IStrStrEntry));
      Dec(Bucket.Count);
      Exit;
    end;
end;

function TIntfIntfHashMap.Size: Integer;
begin
  Result := FCount;
end;

function TIntfIntfHashMap.Values: IIntfCollection;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TIntfArrayList.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J].GetValue);
end;

{ TStrIntfHashMap }

procedure TStrIntfHashMap.Clear;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
      FBuckets[I].Entries[J] := nil;
    FBuckets[I].Count := 0;
  end;
  FCount := 0;
end;

function TStrIntfHashMap.Clone: IInterface;
var
  I, J: Integer;
  NewEntryArray: TStrIntfEntryArray;
  NewMap: TStrIntfHashMap;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  NewMap := TStrIntfHashMap.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
  begin
    NewEntryArray := NewMap.FBuckets[I].Entries;
    SetLength(NewEntryArray, Length(FBuckets[I].Entries));
    for J := 0 to FBuckets[I].Count - 1 do
      NewEntryArray[J] := TStrIntfEntry.Create(FBuckets[I].Entries[J].GetKey,
      	FBuckets[I].Entries[J].GetValue);
    NewMap.FBuckets[I].Count := FBuckets[I].Count;
  end;
  Result := NewMap;
end;

function TStrIntfHashMap.ContainsKey(const Key: string): Boolean;
var
  I: Integer;
  Bucket: PStrIntfBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Key = '' then
    Exit;
  Bucket := @(FBuckets[FHashFunction(HashString(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := True;
      Exit;
    end;
end;

function TStrIntfHashMap.ContainsValue(Value: IInterface): Boolean;
var
  I, J: Integer;
  Bucket: PStrIntfBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Value = nil then
    Exit;
  for J := 0 to FCapacity - 1 do
  begin
    Bucket := @(FBuckets[J]);
    for I := 0 to Bucket.Count - 1 do
      if Bucket.Entries[I].GetValue = Value then
      begin
        Result := True;
        Exit;
      end;
  end;
end;

constructor TStrIntfHashMap.Create;
begin
  Create(16);
end;

constructor TStrIntfHashMap.Create(Capacity: Integer);
var
  I: Integer;
begin
  inherited Create;
  FCapacity := Capacity;
  SetLength(FBuckets, FCapacity);
  for I := 0 to FCapacity - 1 do
    SetLength(FBuckets[I].Entries, 1);
  FHashFunction := HashMul;
end;

destructor TStrIntfHashMap.Destroy;
begin
  Clear;
  inherited;
end;

function TStrIntfHashMap.EntrySet: IIntfSet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TIntfArraySet.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J]);
end;

function TStrIntfHashMap.Equals(AMap: IStrIntfMap): Boolean;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if AMap = nil then
    Exit;
  if FCount <> AMap.Size then
    Exit;
  Result := True;
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
      if AMap.ContainsKey(FBuckets[I].Entries[J].GetKey) then
      begin
        if not (AMap.GetValue(FBuckets[I].Entries[J].GetKey) = FBuckets[I].Entries[J].GetValue) then
        begin
          Result := False;
          Exit;
        end;
      end
      else
      begin
        Result := False;
        Exit;
      end;
  end;
end;

function TStrIntfHashMap.GetValue(const Key: string): IInterface;
var
  I: Integer;
  Index: Integer;
  Bucket: PStrIntfBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := nil;
  if Key = '' then
    Exit;
  Index := FHashFunction(HashString(Key));
  Bucket := @(FBuckets[Index]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := Bucket.Entries[I].GetValue;
      Exit;
    end;
end;

procedure TStrIntfHashMap.GrowEntries(BucketIndex: Integer);
var
  Capacity: Integer;
begin
  Capacity := Length(FBuckets[BucketIndex].Entries);
  if Capacity > 64 then
	  Capacity := Capacity + Capacity div 4
  else
    Capacity := Capacity * 4;
  SetLength(FBuckets[BucketIndex].Entries, Capacity);
end;

function TStrIntfHashMap.HashMul(Key: Integer): Integer;
const
  A = 0.6180339887; // (sqrt(5) - 1) / 2
begin
  Result := Trunc(FCapacity * (Frac(Key * A)));
end;

function TStrIntfHashMap.HashString(const Key: string): Integer;
var
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := 0;
  for I := 1 to Length(Key) do
    Result := Result + (Ord(Key[I]) * (I - 1) * 256);
end;

function TStrIntfHashMap.IsEmpty: Boolean;
begin
  Result := FCount = 0;
end;

function TStrIntfHashMap.KeySet: IStrSet;
var
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
  I, J: Integer;
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TStrArraySet.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
	  	Result.Add(FBuckets[I].Entries[J].Key);
end;

procedure TStrIntfHashMap.PutAll(AMap: IStrIntfMap);
var
  It: IIntfIterator;
  Entry: IStrIntfEntry;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if AMap = nil then
    Exit;
  It := AMap.EntrySet.First;
  while It.HasNext do
  begin
    Entry := IStrIntfEntry(It.Next);
    PutValue(Entry.GetKey, Entry.GetValue);
  end;
end;

procedure TStrIntfHashMap.PutValue(const Key: string; Value: IInterface);
var
  Index: Integer;
  Bucket: PStrIntfBucket;
  Temp: IStrIntfEntry;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if Key = '' then
    Exit;
  if Value = nil then
    Exit;
  Index := FHashFunction(HashString(Key));
  Bucket := @(FBuckets[Index]);
  for I := 0 to Bucket.Count - 1 do
  	if Bucket.Entries[I].Key = Key then
    begin
			Bucket.Entries[I].Value := Value;
      Exit;
    end;
  if Bucket.Count = Length(Bucket.Entries) then
    GrowEntries(Index);
  Temp := TStrIntfEntry.Create(Key, Value);
  Bucket.Entries[Bucket.Count] := Temp;
  Inc(Bucket.Count);
  Inc(FCount);
end;

function TStrIntfHashMap.Remove(const Key: string): IInterface;
var
  Bucket: PStrIntfBucket;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := nil;
  if Key = '' then
    Exit;
  Bucket := @(FBuckets[FHashFunction(HashString(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := Bucket.Entries[I];
      System.Move(Bucket.Entries[I + 1], Bucket.Entries[I],
      	(Bucket.Count - I) * SizeOf(IStrStrEntry));
      Dec(Bucket.Count);
      Exit;
    end;
end;

function TStrIntfHashMap.Size: Integer;
begin
  Result := FCount;
end;

function TStrIntfHashMap.Values: IIntfCollection;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TIntfArrayList.Create;
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J].GetValue);
end;

{ TStrStrHashMap }

procedure TStrStrHashMap.Clear;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
      FBuckets[I].Entries[J] := nil;
    FBuckets[I].Count := 0;
  end;
  FCount := 0;
end;

function TStrStrHashMap.Clone: IInterface;
var
  I, J: Integer;
  NewEntryArray: TStrStrEntryArray;
  NewMap: TStrStrHashMap;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  NewMap := TStrStrHashMap.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
  begin
    NewEntryArray := NewMap.FBuckets[I].Entries;
    SetLength(NewEntryArray, Length(FBuckets[I].Entries));
    for J := 0 to FBuckets[I].Count - 1 do
      NewEntryArray[J] := TStrStrEntry.Create(FBuckets[I].Entries[J].GetKey,
      	FBuckets[I].Entries[J].GetValue);
    NewMap.FBuckets[I].Count := FBuckets[I].Count;
  end;
  Result := NewMap;
end;

function TStrStrHashMap.ContainsKey(const Key: string): Boolean;
var
  I: Integer;
  Bucket: PStrStrBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Key = '' then
    Exit;
  Bucket := @(FBuckets[FHashFunction(HashString(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := True;
      Exit;
    end;
end;

constructor TStrStrHashMap.Create;
begin
  Create(16);
end;

function TStrStrHashMap.ContainsValue(const Value: string): Boolean;
var
  I, J: Integer;
  Bucket: PStrStrBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Value = '' then
    Exit;
  for J := 0 to FCapacity - 1 do
  begin
    Bucket := @(FBuckets[J]);
    for I := 0 to Bucket.Count - 1 do
      if Bucket.Entries[I].GetValue = Value then
      begin
        Result := True;
        Exit;
      end;
  end;
end;

constructor TStrStrHashMap.Create(Capacity: Integer);
var
  I: Integer;
begin
  inherited Create;
  FCapacity := Capacity;
  SetLength(FBuckets, FCapacity);
  for I := 0 to FCapacity - 1 do
    SetLength(FBuckets[I].Entries, 1);
  FHashFunction := HashMul;
end;

function TStrStrHashMap.EntrySet: IIntfSet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TIntfArraySet.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J])
end;

function TStrStrHashMap.Equals(AMap: IStrStrMap): Boolean;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if AMap = nil then
    Exit;
  if FCount <> AMap.Size then
    Exit;
  Result := True;
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
      if AMap.ContainsKey(FBuckets[I].Entries[J].GetKey) then
      begin
        if not (AMap.GetValue(FBuckets[I].Entries[J].GetKey) = FBuckets[I].Entries[J].GetValue) then
        begin
          Result := False;
          Exit;
        end;
      end
      else
      begin
        Result := False;
        Exit;
      end;
  end;
end;

function TStrStrHashMap.GetValue(const Key: string): string;
var
  I: Integer;
  Bucket: PStrStrBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := '';
  if Key = '' then
    Exit;
  Bucket := @(FBuckets[FHashFunction(HashString(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := Bucket.Entries[I].GetValue;
      Exit;
    end;
end;

procedure TStrStrHashMap.GrowEntries(BucketIndex: Integer);
var
  Capacity: Integer;
begin
  Capacity := Length(FBuckets[BucketIndex].Entries);
  if Capacity > 64 then
	  Capacity := Capacity + Capacity div 4
  else
    Capacity := Capacity * 4;
  SetLength(FBuckets[BucketIndex].Entries, Capacity);
end;

function TStrStrHashMap.HashMul(Key: Integer): Integer;
const
  A = 0.6180339887; // (sqrt(5) - 1) / 2
begin
  Result := Trunc(FCapacity * (Frac(Key * A)));
end;

function TStrStrHashMap.IsEmpty: Boolean;
begin
  Result := FCount = 0;
end;

function TStrStrHashMap.KeySet: IStrSet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TStrArraySet.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
	  	Result.Add(FBuckets[I].Entries[J].GetKey)
end;

procedure TStrStrHashMap.PutAll(AMap: IStrStrMap);
var
  It: IIntfIterator;
  Entry: IStrStrEntry;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if AMap = nil then
    Exit;
  It := AMap.EntrySet.First;
  while It.HasNext do
  begin
    Entry := IStrStrEntry(It.Next);
    PutValue(Entry.GetKey, Entry.GetValue);
  end;
end;

procedure TStrStrHashMap.PutValue(const Key, Value: string);
var
  Index: Integer;
  Bucket: PStrStrBucket;
  Temp: IStrStrEntry;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if Key = '' then
    Exit;
  if Value = '' then
    Exit;
  Index := FHashFunction(HashString(Key));
  Bucket := @(FBuckets[Index]);
  for I := 0 to Bucket.Count - 1 do
  	if Bucket.Entries[I].Key = Key then
    begin
			Bucket.Entries[I].Value := Value;
      Exit;
    end;
  if Bucket.Count = Length(Bucket.Entries) then
    GrowEntries(Index);
  Temp := TStrStrEntry.Create(Key, Value);
  Bucket.Entries[Bucket.Count] := Temp;
  Inc(Bucket.Count);
  Inc(FCount);
end;

function TStrStrHashMap.Remove(const Key: string): string;
var
  Bucket: PStrStrBucket;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := '';
  if Key = '' then
    Exit;
  Bucket := @(FBuckets[FHashFunction(HashString(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].GetKey = Key then
    begin
      Result := Bucket.Entries[I].Value;
      System.Move(Bucket.Entries[I + 1], Bucket.Entries[I],
      	(Bucket.Count - I) * SizeOf(IStrStrEntry));
      Dec(Bucket.Count);
      Exit;
    end;
end;

function TStrStrHashMap.Size: Integer;
begin
  Result := FCount;
end;

function TStrStrHashMap.Values: IStrCollection;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TStrArrayList.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J].GetValue);
end;

destructor TStrStrHashMap.Destroy;
begin
  Clear;
  inherited;
end;

function TStrStrHashMap.HashString(const Key: string): Integer;
var
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := 0;
  for I := 1 to Length(Key) do
    Result := Result + (Ord(Key[I]) * (I - 1) * 256);
end;

{ TStrHashMap }

procedure TStrHashMap.Clear;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
    begin
      FBuckets[I].Entries[J].Free;
      FBuckets[I].Entries[J] := nil;
    end;
    FBuckets[I].Count := 0;
  end;
  FCount := 0;
end;

function TStrHashMap.Clone: TObject;
var
  I, J: Integer;
  NewEntryArray: TStrEntryArray;
  NewMap: TStrHashMap;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  NewMap := TStrHashMap.Create(FCapacity, FOwnsObjects);
  for I := 0 to FCapacity - 1 do
  begin
    NewEntryArray := NewMap.FBuckets[I].Entries;
    SetLength(NewEntryArray, Length(FBuckets[I].Entries));
    for J := 0 to FBuckets[I].Count - 1 do
      NewEntryArray[J] := TStrEntry.Create(FBuckets[I].Entries[J].Key,
      	FBuckets[I].Entries[J].Value, FOwnsObjects);
    NewMap.FBuckets[I].Count := FBuckets[I].Count;
  end;
  Result := NewMap;
end;

function TStrHashMap.ContainsKey(const Key: string): Boolean;
var
  I: Integer;
  Bucket: PStrBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Key = '' then
    Exit;
  Bucket := @(FBuckets[FHashFunction(HashString(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].Key = Key then
    begin
      Result := True;
      Exit;
    end;
end;

function TStrHashMap.ContainsValue(Value: TObject): Boolean;
var
  I, J: Integer;
  Bucket: PStrBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Value = nil then
    Exit;
  for J := 0 to FCapacity - 1 do
  begin
    Bucket := @(FBuckets[J]);
    for I := 0 to Bucket.Count - 1 do
      if Bucket.Entries[I].Value = Value then
      begin
        Result := True;
        Exit;
      end;
  end;
end;

constructor TStrHashMap.Create;
begin
  Create(16, True);
end;

constructor TStrHashMap.Create(Capacity: Integer; AOwnsObjects: Boolean);
var
  I: Integer;
begin
  inherited Create;
  FCapacity := Capacity;
  FOwnsObjects := AOwnsObjects;
  SetLength(FBuckets, FCapacity);
  for I := 0 to FCapacity - 1 do
    SetLength(FBuckets[I].Entries, 1);
  FHashFunction := HashMul;
end;

destructor TStrHashMap.Destroy;
begin
  Clear;
  inherited;
end;

function TStrHashMap.EntrySet: ISet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TArraySet.Create(FCapacity, False);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J]);
end;

function TStrHashMap.Equals(AMap: IStrMap): Boolean;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if AMap = nil then
    Exit;
  if FCount <> AMap.Size then
    Exit;
  Result := True;
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
      if AMap.ContainsKey(FBuckets[I].Entries[J].Key) then
      begin
        if not (AMap.GetValue(FBuckets[I].Entries[J].Key) = FBuckets[I].Entries[J].Value) then
        begin
          Result := False;
          Exit;
        end;
      end
      else
      begin
        Result := False;
        Exit;
      end;
  end;
end;

function TStrHashMap.GetValue(const Key: string): TObject;
var
  I: Integer;
  Bucket: PStrBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := nil;
  if Key = '' then
    Exit;
  I := FHashFunction(HashString(Key));
  Bucket := @(FBuckets[I]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].Key = Key then
    begin
      Result := Bucket.Entries[I].Value;
      Exit;
    end;
end;

procedure TStrHashMap.GrowEntries(BucketIndex: Integer);
var
  Capacity: Integer;
begin
  Capacity := Length(FBuckets[BucketIndex].Entries);
  if Capacity > 64 then
	  Capacity := Capacity + Capacity div 4
  else
    Capacity := Capacity * 4;
  SetLength(FBuckets[BucketIndex].Entries, Capacity);
end;

function TStrHashMap.HashMul(Key: Integer): Integer;
const
  A = 0.6180339887; // (sqrt(5) - 1) / 2
begin
  Result := Trunc(FCapacity * (Frac(Key * A)));
//  Result := LongRec(Key).Bytes[1] and $FF;
end;

function TStrHashMap.HashString(const Key: string): Integer;
var
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := 0;
  for I := 1 to Length(Key) do
    Result := Result + (Ord(Key[I]) * (I - 1) * 256);
end;

function TStrHashMap.IsEmpty: Boolean;
begin
  Result := FCount = 0;
end;

function TStrHashMap.KeySet: IStrSet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TStrArraySet.Create(FCapacity);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
	  	Result.Add(FBuckets[I].Entries[J].Key);
end;

procedure TStrHashMap.PutAll(AMap: IStrMap);
var
  It: IIterator;
  Entry: TStrEntry;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if AMap = nil then
    Exit;
  It := AMap.EntrySet.First;
  while It.HasNext do
  begin
    Entry := TStrEntry(It.Next);
    PutValue(Entry.Key, Entry.Value);
  end;
end;

procedure TStrHashMap.PutValue(const Key: string; Value: TObject);
var
  Index: Integer;
  Bucket: PStrBucket;
  Temp: TStrEntry;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if Key = '' then
    Exit;
  if Value = nil then
    Exit;
  Index := FHashFunction(HashString(Key));
  Bucket := @(FBuckets[Index]);
  for I := 0 to Bucket.Count - 1 do
  	if Bucket.Entries[I].Key = Key then
    begin
			Bucket.Entries[I].Value := Value;
      Exit;
    end;
  if Bucket.Count = Length(Bucket.Entries) then
    GrowEntries(Index);
  Temp := TStrEntry.Create(Key, Value, FOwnsObjects);
  Bucket.Entries[Bucket.Count] := Temp;
  Inc(Bucket.Count);
  Inc(FCount);
end;

function TStrHashMap.Remove(const Key: string): TObject;
var
  Bucket: PStrBucket;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := nil;
  if Key = '' then
    Exit;
  Bucket := @(FBuckets[FHashFunction(HashString(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].Key = Key then
    begin
      if not FOwnsObjects then
	      Result := Bucket.Entries[I].FValue;
      Bucket.Entries[I].Free;
      Bucket.Entries[I] := nil;
      System.Move(Bucket.Entries[I + 1], Bucket.Entries[I],
      	(Bucket.Count - I) * SizeOf(TStrEntry));
      Dec(Bucket.Count);
      Exit;
    end;
end;

function TStrHashMap.Size: Integer;
begin
  Result := FCount;
end;

function TStrHashMap.Values: ICollection;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TArrayList.Create(FCapacity, False); // NEVER Owns Objects !
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J].Value);
end;

{ THashMap }

procedure THashMap.Clear;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
    begin
      FBuckets[I].Entries[J].Free;
      FBuckets[I].Entries[J] := nil;
    end;
    FBuckets[I].Count := 0;
  end;
  FCount := 0;
end;

function THashMap.Clone: TObject;
var
  I, J: Integer;
  NewEntryArray: TEntryArray;
  NewMap: THashMap;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  NewMap := THashMap.Create(FCapacity, FOwnsObjects);
  for I := 0 to FCapacity - 1 do
  begin
    NewEntryArray := NewMap.FBuckets[I].Entries;
    SetLength(NewEntryArray, Length(FBuckets[I].Entries));
    for J := 0 to FBuckets[I].Count - 1 do
      NewEntryArray[J] := TEntry.Create(FBuckets[I].Entries[J].Key,
      	FBuckets[I].Entries[J].Value, FOwnsObjects);
    NewMap.FBuckets[I].Count := FBuckets[I].Count;
  end;
  Result := NewMap;
end;

function THashMap.ContainsKey(Key: TObject): Boolean;
var
  I: Integer;
  Bucket: PBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Key = nil then
    Exit;
  Bucket := @(FBuckets[FHashFunction(Integer(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].Key = Key then
    begin
      Result := True;
      Exit;
    end;
end;

function THashMap.ContainsValue(Value: TObject): Boolean;
var
  I, J: Integer;
  Bucket: PBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if Value = nil then
    Exit;
  for J := 0 to FCapacity - 1 do
  begin
    Bucket := @(FBuckets[J]);
    for I := 0 to Bucket.Count - 1 do
      if Bucket.Entries[I].Value = Value then
      begin
        Result := True;
        Exit;
      end;
  end;
end;

constructor THashMap.Create;
begin
  Create(16, True);
end;

constructor THashMap.Create(Capacity: Integer; AOwnsObjects: Boolean);
var
  I: Integer;
begin
  inherited Create;
  FCapacity := Capacity;
  FOwnsObjects := AOwnsObjects;
  SetLength(FBuckets, FCapacity);
  for I := 0 to FCapacity - 1 do
    SetLength(FBuckets[I].Entries, 1);
  FHashFunction := HashMul;
end;

destructor THashMap.Destroy;
begin
  Clear;
  inherited;
end;

function THashMap.EntrySet: ISet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TArraySet.Create(FCapacity, False);
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J]);
end;

function THashMap.Equals(AMap: IMap): Boolean;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := False;
  if AMap = nil then
    Exit;
  if FCount <> AMap.Size then
    Exit;
  Result := True;
  for I := 0 to FCapacity - 1 do
  begin
    for J := 0 to FBuckets[I].Count - 1 do
      if AMap.ContainsKey(FBuckets[I].Entries[J].Key) then
      begin
        if not (AMap.GetValue(FBuckets[I].Entries[J].Key) = FBuckets[I].Entries[J].Value) then
        begin
          Result := False;
          Exit;
        end;
      end
      else
      begin
        Result := False;
        Exit;
      end;
  end;
end;

function THashMap.GetValue(Key: TObject): TObject;
var
  I: Integer;
  Bucket: PBucket;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := nil;
  if Key = nil then
    Exit;
  Bucket := @(FBuckets[FHashFunction(Integer(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].Key = Key then
    begin
      Result := Bucket.Entries[I].Value;
      Exit;
    end;
end;

procedure THashMap.GrowEntries(BucketIndex: Integer);
var
  Capacity: Integer;
begin
  Capacity := Length(FBuckets[BucketIndex].Entries);
  if Capacity > 64 then
	  Capacity := Capacity + Capacity div 4
  else
    Capacity := Capacity * 4;
  SetLength(FBuckets[BucketIndex].Entries, Capacity);
end;

function THashMap.HashMul(Key: Integer): Integer;
const
  A = 0.6180339887; // (sqrt(5) - 1) / 2
begin
  Result := Trunc(FCapacity * (Frac(Key * A)));
//  Result := LongRec(Key).Bytes[1] and $FF;
end;

function THashMap.IsEmpty: Boolean;
begin
  Result := FCount = 0;
end;

function THashMap.KeySet: ISet;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TArraySet.Create(FCapacity, False); // NEVER Owns Objects !
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
	  	Result.Add(FBuckets[I].Entries[J].Key);
end;

procedure THashMap.PutAll(AMap: IMap);
var
  It: IIterator;
  Entry: TEntry;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if AMap = nil then
    Exit;
  It := AMap.EntrySet.First;
  while It.HasNext do
  begin
    Entry := TEntry(It.Next);
    PutValue(Entry.Key, Entry.Value);
  end;
end;

procedure THashMap.PutValue(Key, Value: TObject);
var
  Index: Integer;
  Bucket: PBucket;
  Temp: TEntry;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  if Key = nil then
    Exit;
  if Value = nil then
    Exit;
  Index := FHashFunction(Integer(Key));
  Bucket := @(FBuckets[Index]);
  for I := 0 to Bucket.Count - 1 do
  	if Bucket.Entries[I].Key = Key then
    begin
			Bucket.Entries[I].Value := Value;
      Exit;
    end;
  if Bucket.Count = Length(Bucket.Entries) then
    GrowEntries(Index);
  Temp := TEntry.Create(Key, Value, FOwnsObjects);
  Bucket.Entries[Bucket.Count] := Temp;
  Inc(Bucket.Count);
  Inc(FCount);
end;

function THashMap.Remove(Key: TObject): TObject;
var
  Bucket: PBucket;
  I: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := nil;
  if Key = nil then
    Exit;
  Bucket := @(FBuckets[FHashFunction(Integer(Key))]);
  for I := 0 to Bucket.Count - 1 do
    if Bucket.Entries[I].Key = Key then
    begin
      if not FOwnsObjects then
        Result := Bucket.Entries[I].FValue;
      Bucket.Entries[I].Free;
      Bucket.Entries[I] := nil;
      System.Move(Bucket.Entries[I + 1], Bucket.Entries[I],
      	(Bucket.Count - I) * SizeOf(TEntry));
      Dec(Bucket.Count);
      Exit;
    end;
end;

function THashMap.Size: Integer;
begin
  Result := FCount;
end;

function THashMap.Values: ICollection;
var
  I, J: Integer;
{$IFDEF THREADSAFE}
  CS: IInterface;
{$ENDIF}
begin
{$IFDEF THREADSAFE}
  CS := EnterCriticalSection;
{$ENDIF}
  Result := TArrayList.Create(FCapacity, False); // NEVER Owns Objects !
  for I := 0 to FCapacity - 1 do
    for J := 0 to FBuckets[I].Count - 1 do
      Result.Add(FBuckets[I].Entries[J].Value);
end;


end.
