unit WstWizard;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  StdCtrls, ExtCtrls, Forms, ComCtrls, Grids, DBGrids,
  WWDataModule, SIEDataModule, DBCtrls, RXDBCtrl;

type

  TWstType = (wtNone, wtHospital, wtBrigade);

  TAdminWizard = class(TForm)
	pcWstWiz: TPageControl;
	tsHospital: TTabSheet;
	tsWorkstation: TTabSheet;
	tsOperator: TTabSheet;
	BtmPanel: TPanel;
	BtnPanel: TPanel;
	PrevButton: TButton;
	NextButton: TButton;
	CancelButton: TButton;
	gbHospital: TGroupBox;
	gbWorkstation: TGroupBox;
	grdWorkstation: TRxDBGrid;
	wstBtnPanel: TPanel;
	wstAddButton: TButton;
	gbOperator: TGroupBox;
	grdOperator: TRxDBGrid;
	userBtnPanel: TPanel;
	userAddButton: TButton;
	userPwdButton: TButton;
	grdHospital: TRxDBGrid;
	wstSendButton: TButton;
	userEdtButton: TButton;
	tsRegion: TTabSheet;
    gbRegion: TGroupBox;
    grdRegion: TRxDBGrid;
    tsWstType: TTabSheet;
    gbWstType: TGroupBox;
	rbHospital: TRadioButton;
	rbBrigade: TRadioButton;
    tsBrigade: TTabSheet;
    gbBrigade: TGroupBox;
    grdBrigade: TRxDBGrid;
    brgBtnPanel: TPanel;
	brgAddButton: TButton;
    brgEdtButton: TButton;
    tsCity: TTabSheet;
    gbCity: TGroupBox;
    grdCity: TRxDBGrid;
    Panel1: TPanel;
	Label1: TLabel;
	edRegionFilter: TEdit;
    Panel2: TPanel;
    Label2: TLabel;
    edHospitalFilter: TEdit;
    Panel3: TPanel;
    Label3: TLabel;
    edCityFilter: TEdit;
    rbRegionFilter: TRadioGroup;
    rbHospitalFilter: TRadioGroup;
    rbCityFilter: TRadioGroup;
	procedure FormCreate(Sender: TObject);
	procedure FormDestroy(Sender: TObject);
	procedure NextButtonClick(Sender: TObject);
	procedure PrevButtonClick(Sender: TObject);
	procedure pcWstWizChange(Sender: TObject);
	procedure wstSendButtonClick(Sender: TObject);
	procedure userPwdButtonClick(Sender: TObject);
	procedure userAddButtonClick(Sender: TObject);
	procedure userEdtButtonClick(Sender: TObject);
	procedure wstAddButtonClick(Sender: TObject);
    procedure rbHospitalClick(Sender: TObject);
    procedure brgAddButtonClick(Sender: TObject);
    procedure brgEdtButtonClick(Sender: TObject);
    procedure edRegionFilterChange(Sender: TObject);
	procedure edHospitalFilterChange(Sender: TObject);
    procedure edCityFilterChange(Sender: TObject);
  private
	WWDM : TWWDM;
	SIEDM : TSIEDM;
	function OperatorDlgShow(const GID: Variant) : TModalResult;
	function WorkstationDlgShow(const GID: Variant) : TModalResult;
	function BrigadeDlgShow(const GID: Variant) : TModalResult;
	procedure ExportScript(const Script : TStrings);
	procedure CreateDeleteUserScript(Script : TStrings);
	procedure CreateUpdateUserScript(Script : TStrings);
	procedure CreateInsertUserScript(Script : TStrings);
	function WstType : TWstType;
  public
	procedure CheckControls;
  end;

  TSimpleFilter = class
  public
	constructor Create(FilterString : String);
	destructor Destroy; override;
	function ToFilterString : string;
	procedure PutValue(const Key, Value : String);
	procedure Remove(const Key : String);
  private
	_Entries : TList;
  end;


implementation

uses    dialogs, filectrl, db,
		// RX units
		FileUtil,RxMemDS,
		// User units
		Replications, Unit_GlobalObjects, StrConst,
		form_Operator, form_Brigade, form_Workstation;

{$R *.DFM}

type
  TStrStrEntry = record
	Key, Value : String;
  end;

constructor TSimpleFilter.Create(FilterString : String);
var p : Integer;

	procedure parseToken(const token : string);
	var i : Integer;
	begin
		i := pos('=',token);
		if i=0 then
			i := pos(' ',token);
		PutValue(Copy(token,1,i-1),Copy(token,i,length(token)));
	end;

begin
	inherited Create;
	_Entries := TList.Create;
	if FilterString<>'' then
	begin
		while true do
		begin
			p := Pos(' AND ', FilterString);
			if p = 0 then break;
			parseToken(Copy(FilterString, 1, p-1));
			Delete(FilterString,1,p+4);
		end;
		parseToken(FilterString);
	end;
end;

destructor TSimpleFilter.Destroy;
var
	entry : ^TStrStrEntry;
begin
	while _Entries.Count>0 do
	begin
		entry := _Entries.Items[0];
		_Entries.Remove(entry);
		Dispose(entry);
	end;
	inherited;
end;

procedure TSimpleFilter.PutValue(const Key, Value : String);
var
	entry : ^TStrStrEntry;
	i : Integer;
begin
  For i := 1 to _Entries.Count do
  begin
	  entry := _Entries.Items[i-1];
	  if entry^.Key = Key then
	  begin
		entry^.Value := Value;
		Exit;
	  end;
  end;
  New(entry);
  entry^.Key := Key;
  entry^.Value := Value;
  _Entries.Add(entry);
end;

procedure TSimpleFilter.Remove(const Key : String);
var
	entry : ^TStrStrEntry;
	i : Integer;
begin
  For i := 1 to _Entries.Count do
  begin
	  entry := _Entries.Items[i-1];
	  if entry^.Key = Key then
	  begin
		_Entries.Remove(entry);
		Dispose(entry);
		Exit;
	  end;
  end;
end;

function TSimpleFilter.ToFilterString : string;
var
	entry : ^TStrStrEntry;
	i : Integer;
begin
	Result := '';
	if _Entries.Count>0 then
	begin
	  For i := 1 to _Entries.Count do
	  begin
		  entry := _Entries.Items[i-1];
		  Result := Result + entry^.Key + entry^.Value + ' AND '
	  end;
	  Delete(Result, length(Result)-4, 5);
	end;
end;

procedure TAdminWizard.FormCreate(Sender: TObject);
begin
	WWDM := TWWDM.Create(self);
	SIEDM := TSIEDM.Create(self);
	pcWstWiz.ActivePageIndex := 0;
	WWDM.itRegionNotNull.Open;
	WWDM.itRegion.Open;
	WWDM.itCity.Open;
	WWDM.itHospital.Open;
	WWDM.itWorkStation.Open;
	WWDM.itOperator.Open;
	WWDM.itRegionAfterScroll(WWDM.itRegion);
end;

procedure TAdminWizard.FormDestroy(Sender: TObject);
begin
	WWDM.itOperator.Close;
	WWDM.itWorkStation.Close;
	WWDM.itHospital.Close;
	WWDM.itRegion.Close;
	WWDM.itRegionNotNull.Close;
	FreeAndNil(WWDM);
	FreeAndNil(SIEDM);
end;

procedure TAdminWizard.NextButtonClick(Sender: TObject);
var PageIndex : Integer;
begin
	PageIndex := pcWstWiz.ActivePageIndex;
	repeat
		PageIndex := PageIndex+1;
	until pcWstWiz.Pages[PageIndex].TabVisible;
	pcWstWiz.ActivePageIndex := PageIndex;
	CheckControls;
end;

procedure TAdminWizard.PrevButtonClick(Sender: TObject);
var PageIndex : Integer;
begin
	PageIndex := pcWstWiz.ActivePageIndex;
	repeat
		PageIndex := PageIndex-1;
	until pcWstWiz.Pages[PageIndex].TabVisible;
	pcWstWiz.ActivePageIndex := PageIndex;
	CheckControls;
end;

procedure TAdminWizard.CheckControls;
var WstTypeSelected,
	RegionSelected,
	CitySelected,
	tsWorkstationEnabled,
	tsOperatorEnabled,
	OperatorSelected : Boolean;
begin

	WstTypeSelected := (WstType<>wtNone);

	if grdRegion.DataSource.DataSet.Active then
		RegionSelected := (grdRegion.DataSource.DataSet.RecordCount>0)
	else
		RegionSelected := False;

	if grdCity.DataSource.DataSet.Active then
		CitySelected := (grdCity.DataSource.DataSet.RecordCount>0)
	else
		CitySelected := False;

	if (WstType=wtHospital) then
		if grdHospital.DataSource.DataSet.Active then
			tsWorkstationEnabled := (grdHospital.DataSource.DataSet.RecordCount>0)
		else
			tsWorkstationEnabled := False
	else if (WstType=wtBrigade) then
		if grdBrigade.DataSource.DataSet.Active then
			tsWorkstationEnabled := (grdBrigade.DataSource.DataSet.RecordCount>0)
		else
			tsWorkstationEnabled := False
	else tsWorkstationEnabled := False;

	if grdWorkstation.DataSource.DataSet.Active then
		tsOperatorEnabled := (grdWorkstation.DataSource.DataSet.RecordCount>0)
	else
		tsOperatorEnabled := False;

	if grdOperator.DataSource.DataSet.Active then
		OperatorSelected := (grdOperator.DataSource.DataSet.RecordCount>0)
	else
		OperatorSelected := False;

	tsRegion.TabVisible := WstTypeSelected;
	tsCity.TabVisible := RegionSelected and (WstType=wtBrigade);
	tsHospital.TabVisible := RegionSelected and (WstType=wtHospital);
	tsBrigade.TabVisible := CitySelected and (WstType=wtBrigade);
	tsWorkstation.TabVisible := tsWorkstationEnabled;// and tsRegionEnabled;
	tsOperator.TabVisible := tsOperatorEnabled;// and tsRegionEnabled;

	case pcWstWiz.ActivePageIndex of
	0:	begin
			NextButton.Enabled := WstTypeSelected;
			PrevButton.Enabled := False;
		end;
	1:	begin
			NextButton.Enabled := RegionSelected;
			PrevButton.Enabled := True;
		end;
	2:	begin
			NextButton.Enabled := tsWorkstationEnabled;
			PrevButton.Enabled := True;
		end;
	3:	begin
			NextButton.Enabled := CitySelected;
			PrevButton.Enabled := True;
		end;
	4:	begin
			NextButton.Enabled := tsWorkstationEnabled;
			PrevButton.Enabled := True;
			brgEdtButton.Enabled := tsWorkstationEnabled;
		end;
	5:	begin
			NextButton.Enabled := tsOperatorEnabled;
			PrevButton.Enabled := True;
//			wstEdtButton.Enabled := tsOperatorEnabled;
			wstSendButton.Enabled := tsOperatorEnabled;
		end;
	6:	begin
			NextButton.Enabled := False;
			PrevButton.Enabled := True;
			userPwdButton.Enabled := OperatorSelected;
			userEdtButton.Enabled := OperatorSelected;
		end;
	end;
end;

procedure TAdminWizard.pcWstWizChange(Sender: TObject);
begin
	CheckControls;
end;

procedure TAdminWizard.wstSendButtonClick(Sender: TObject);
begin
	TReplicator.ExportWorkStation(
		grdWorkstation.DataSource.DataSet.FieldValues['WstGID']
	);
end;

procedure TAdminWizard.userPwdButtonClick(Sender: TObject);
var	SQL : TStringList;
begin
	SQL := TStringList.Create;
	try
		SQL.Append('UPDATE Operator');
		SQL.Append('SET');
		SQL.Append('	UserHash=''0''');
		SQL.Append('WHERE');
		SQL.Append('	UserGID=' +
			grdOperator.DataSource.DataSet.FieldByName('UserGID').AsString);
		SQL.Append('	AND');
		SQL.Append('	WstGID=' +
			grdOperator.DataSource.DataSet.FieldByName('WstGID').AsString+';');
		ExportScript(SQL);
	finally
		FreeAndNil(SQL);
	end;
end;

procedure TAdminWizard.userAddButtonClick(Sender: TObject);
var	SQL : TStringList;
begin
	if OperatorDlgShow(null)=mrOk then
	begin
		SQL := TStringList.Create;
		try
			CreateInsertUserScript(SQL);
			ExportScript(SQL);
		finally
			FreeAndNil(SQL);
		end;
	end;
end;

function TAdminWizard.OperatorDlgShow(const GID: Variant) : TModalResult;
var
  OperatorDlg: TOperatorDlg;
begin
	SIEDM.itOperator.Open;
	try
		OperatorDlg := TOperatorDlg.Create(
			self,
			SIEDM.itOperator,
			GID,
			grdWorkstation.DataSource.DataSet.FieldValues['WstGID']);
		try
			Result := OperatorDlg.ShowModal;
		finally
			grdOperator.DataSource.DataSet.Close;
			grdOperator.DataSource.DataSet.Open;
			grdOperator.DataSource.DataSet.Locate(
				'UserGID;WstGID',
				VarArrayOf([SIEDM.itOperator.FieldValues['UserGID'],
							SIEDM.itOperator.FieldValues['WstGID']]),
				[]
			);
			CheckControls;
			OperatorDlg.Free;
		end;
	finally
		SIEDM.itOperator.Close;
	end;
end;

procedure TAdminWizard.userEdtButtonClick(Sender: TObject);
var	SQL : TStringList;
begin
	SQL := TStringList.Create;
	CreateDeleteUserScript(SQL);
	try
		case OperatorDlgShow(grdOperator.DataSource.DataSet.FieldValues['UserGID']) of
		mrOK :
			begin
				SQL.Clear;
				CreateUpdateUserScript(SQL);
				ExportScript(SQL);
			end;
		mrAbort :
			ExportScript(SQL);
		end;
	finally
		FreeAndNil(SQL);
	end;
end;

procedure TAdminWizard.ExportScript(const Script: TStrings);
var	TargetName, FileName, RplFolder, FmtDateTime : String;
	CurrentDate : TDateTime;
begin
	RplFolder := '';
	try
		CurrentDate := Now;

		FmtDateTime := FormatDateTime('yyyymmddhhnnss',CurrentDate);

		FileName :=
			Format('%.6d',[grdOperator.DataSource.DataSet.FieldByName('WstGID').AsInteger])+
			'_' + FmtDateTime+'_ ';

		if (WstType=wtHospital) then
			TargetName := ConvertFileName(grdHospital.DataSource.DataSet.FieldValues['MU'])
		else if (WstType=wtBrigade) then
			TargetName := ConvertFileName(grdBrigade.DataSource.DataSet.FieldValues['BrgName'])
		else raise Exception.Create('');

		RplFolder :=
			AppInfo.ExportDBPath+Trim(Copy(TargetName,1,30));

		if not DirectoryExists(RplFolder) then
			if Not CreateDir(RplFolder) then
				raise Exception.Create(ERR_MSG_CREATE_FOLDER+Chr(13)+RplFolder);

		FileName := FileName + TargetName;

		Script.Insert(0,'*/');
		Script.Insert(0,FileName);
		Script.Insert(0,'/*');

		FileName := Copy(
			ConvertFileName(grdWorkstation.DataSource.DataSet.FieldValues['WSTNAME']),
			1,12) + '_' + FmtDateTime;

		FileName := RplFolder + '\' + FileName + '.sql';

		Script.SaveToFile(FileName);
		ShowMessage(MSG_EXPORT_FILES + RplFolder);
	except
		on E: Exception do
		begin
			if (RplFolder<>'') then
			begin
				DeleteFiles(RplFolder+'\*.*');
				RemoveDirectory(PChar(RplFolder));
			end;
			HandleException(E);
		end;
	end;
end;

procedure TAdminWizard.CreateDeleteUserScript(Script: TStrings);
begin
	with Script do
	begin
		Append('DELETE FROM Operator');
		Append('WHERE');
		Append('	UserGID=' +
			grdOperator.DataSource.DataSet.FieldByName('UserGID').AsString);
		Append('	AND');
		Append('	WstGID=' +
			grdOperator.DataSource.DataSet.FieldByName('WstGID').AsString+';');
	end;
end;

procedure TAdminWizard.CreateInsertUserScript(Script: TStrings);
begin
	with Script do
	begin
		Append('INSERT INTO Operator (WstGID, UserGID, UserName, UserHash, UserLevel)');
		Append('VALUES ('+
			grdOperator.DataSource.DataSet.FieldByName('WstGID').AsString+','+
			grdOperator.DataSource.DataSet.FieldByName('UserGID').AsString+','+
			''''+grdOperator.DataSource.DataSet.FieldByName('UserName').AsString+''','+
			''''+grdOperator.DataSource.DataSet.FieldByName('UserHash').AsString+''','+
			grdOperator.DataSource.DataSet.FieldByName('UserLevel').AsString+');');
	end;
end;

procedure TAdminWizard.CreateUpdateUserScript(Script: TStrings);
begin
	with Script do
	begin
		Append('UPDATE Operator');
		Append('SET');
		Append('	UserName='''+grdOperator.DataSource.DataSet.FieldByName('UserName').AsString+''',');
		Append('	UserHash='''+grdOperator.DataSource.DataSet.FieldByName('UserHash').AsString+''',');
		Append('	UserLevel='+grdOperator.DataSource.DataSet.FieldByName('UserLevel').AsString);
		Append('WHERE');
		Append('	UserGID=' +
			grdOperator.DataSource.DataSet.FieldByName('UserGID').AsString);
		Append('	AND');
		Append('	WstGID=' +
			grdOperator.DataSource.DataSet.FieldByName('WstGID').AsString+';');
	end;
end;

procedure TAdminWizard.wstAddButtonClick(Sender: TObject);
begin
	if WorkstationDlgShow(null)=mrOk then
		if MessageDlg(MSG_SEND_RPL,mtCustom	,[mbYes, mbNo],0)=mrYes then
			wstSendButtonClick(Sender);
end;

function TAdminWizard.WorkstationDlgShow(const GID: Variant): TModalResult;
var
	WorkstationDialog: TWorkstationDialog;
	HspGID, BrgGID: Variant;
begin
	SIEDM.itWorkStation.Open;
	if WstType=wtHospital then
		HspGID := grdHospital.DataSource.DataSet.FieldValues['HspGID']
	else HspGID:=null;
	if WstType=wtBrigade then
		BrgGID := grdBrigade.DataSource.DataSet.FieldValues['BRGGID']
	else BrgGID:=null;
	try
		WorkstationDialog := TWorkstationDialog.Create(
			self,
			SIEDM.itWorkStation,
			GID,
			HspGID,
			BrgGID);
		try
			Result := WorkstationDialog.ShowModal;
		finally
			grdWorkstation.DataSource.DataSet.Close;
			grdWorkstation.DataSource.DataSet.Open;
			grdWorkstation.DataSource.DataSet.Locate(
				'WstGID',
				SIEDM.itWorkStation.FieldValues['WstGID'],
				[]
			);
			CheckControls;
			WorkstationDialog.Free;
		end;
	finally
		SIEDM.itWorkStation.Close;
	end;
end;

function TAdminWizard.WstType : TWstType;
begin
	if rbHospital.Checked then Result := wtHospital
	else if rbBrigade.Checked then Result := wtBrigade
	else Result := wtNone;
end;

procedure TAdminWizard.rbHospitalClick(Sender: TObject);
begin
	if WstType=wtHospital then
	begin
		WWDM.itWorkstation.SQL[2]:='HspGID=:HSPGID';
		WWDM.itWorkstation.DataSource := WWDM.dsHospital;
	end	else if WstType=wtBrigade then
	begin
		WWDM.itWorkstation.SQL[2]:='BrgGID=:BRGGID';
		WWDM.itWorkstation.DataSource := WWDM.dsBrigade;
	end;
	WWDM.itRegionAfterScroll(WWDM.itRegion);
//	WWDM.itWorkstation.Open;
	CheckControls;
end;

procedure TAdminWizard.brgAddButtonClick(Sender: TObject);
begin
	BrigadeDlgShow(null);
end;

procedure TAdminWizard.brgEdtButtonClick(Sender: TObject);
begin
	BrigadeDlgShow(grdBrigade.DataSource.DataSet.FieldValues['BrgGID']);
end;

function TAdminWizard.BrigadeDlgShow(const GID: Variant) : TModalResult;
var
  BrigadeDlg: TBrigadeDlg;
begin
	SIEDM.itBrigade.Open;
	try
		BrigadeDlg := TBrigadeDlg.Create(
			self,
			SIEDM.itBrigade,
			SIEDM.itBrigHsp,
			GID,
			grdRegion.DataSource.DataSet.FieldValues['RegGID'],
			grdCity.DataSource.DataSet.FieldValues['CityGID'],
			WWDM);
		try
			Result := BrigadeDlg.ShowModal;
		finally
			grdBrigade.DataSource.DataSet.Close;
			grdBrigade.DataSource.DataSet.Open;
			grdBrigade.DataSource.DataSet.Locate(
				'BrgGID',
				SIEDM.itBrigade.FieldValues['BrgGID'],
				[]
			);
			CheckControls;
			BrigadeDlg.Free;
		end;
	finally
		SIEDM.itBrigade.Close;
	end;
end;

procedure FilterChange( SrcDataset, DstDataset : TDataSet;
						const FilterText, VarName : String);
var sf : TSimpleFilter;
begin
	SrcDataset.FilterOptions := [foCaseInsensitive];
	sf := TSimpleFilter.Create(SrcDataset.Filter);
	try
		if FilterText='' then
			sf.Remove(VarName)
		else
			sf.PutValue(VarName, '='''+FilterText+'*''');
		SrcDataset.Filter := sf.ToFilterString;
		SrcDataset.Filtered := True;
		if SrcDataset<>DstDataset then
		begin
			DstDataset.Close;
			DstDataset.Open;
		end;
	finally
		sf.Free;
	end;
end;

function ToRegularExpression( const S : String ) : String;
begin
	Result := '%'+StringReplace(S,' ','%',[rfReplaceAll])+'%';
end;

procedure TAdminWizard.edRegionFilterChange(Sender: TObject);
begin
	if rbRegionFilter.ItemIndex=0 then
	begin
		if WWDM.itRegionNotNull.SQL[2]<>'''%''' then
			WWDM.itRegionNotNull.SQL[2] := '''%''';
		FilterChange(WWDM.itRegionNotNull,WWDM.itRegion,edRegionFilter.Text,'REGNAME');
	end	else
	begin
		WWDM.itRegionNotNull.SQL[2] := ''''+ToRegularExpression(edRegionFilter.Text)+'''';
		FilterChange(WWDM.itRegionNotNull,WWDM.itRegion,'','REGNAME');
	end;
end;

procedure TAdminWizard.edHospitalFilterChange(Sender: TObject);
begin
	if rbHospitalFilter.ItemIndex=0 then
	begin
		if WWDM.itHospital.SQL[4]<>'''%''' then
		begin
			WWDM.itHospital.SQL[4] := '''%''';
			WWDM.itHospital.Close;
			WWDM.itHospital.Open;
		end;
		FilterChange(WWDM.itHospital,WWDM.itHospital,edHospitalFilter.Text,'MU');
	end	else
	begin
		WWDM.itHospital.SQL[4] := ''''+ToRegularExpression(edHospitalFilter.Text)+'''';
		FilterChange(WWDM.itHospital,WWDM.itHospital,'','MU');
		WWDM.itHospital.Close;
		WWDM.itHospital.Open;
	end;
end;

procedure TAdminWizard.edCityFilterChange(Sender: TObject);
begin
	if rbCityFilter.ItemIndex=0 then
	begin
		if WWDM.itCityNotNull.SQL[2]<>'''%''' then
			WWDM.itCityNotNull.SQL[2] := '''%''';
		FilterChange(WWDM.itCityNotNull,WWDM.itCity,edCityFilter.Text,'CITYNAME');
	end	else
	begin
		WWDM.itCityNotNull.SQL[2] := ''''+ToRegularExpression(edCityFilter.Text)+'''';
		FilterChange(WWDM.itCityNotNull,WWDM.itCity,'','CITYNAME');
	end;
end;

end.
