unit form_Brigade;

interface

uses 
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Dialogs, Controls, StdCtrls, 
  Buttons, ExtCtrls, dbtables,
  DBObjects, WWDataModule, RXCtrls;

type
  TBrigadeDlg = class(TForm)
	OKBtn: TButton;
	CancelBtn: TButton;
    Bevel1: TBevel;
    edName: TEdit;
    lbName: TLabel;
    gbSelectLPU: TGroupBox;
    SrcLabel: TLabel;
    IncludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    ExAllBtn: TSpeedButton;
    DstLabel: TLabel;
    SrcList: TTextListBox;
    DstList: TTextListBox;
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
  private
	{ Private declarations }
	WWDM : TWWDM;
	HospGIDs : TStringList;
	BrigHosp : TTable;
  public
	{ Public declarations }
	DBObj : TSimpleDBObject;
	constructor Create(AOwner: TComponent; Table, BrigHosp : TTable;
		const BrgGID, RegGID, CityGID : Variant; WWDM: TWWDM); reintroduce;
	destructor Destroy; override;
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
	procedure SetItem(List: TTextListBox; Index: Integer);
	function GetFirstSelection(List: TCustomListBox): Integer;
	procedure SetButtons;
  end;

var
  BrigadeDlg: TBrigadeDlg;

implementation

uses StrConst;

{$R *.DFM}

procedure TBrigadeDlg.IncludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  MoveSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
end;

procedure TBrigadeDlg.ExcludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  MoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
end;

procedure TBrigadeDlg.IncAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to SrcList.Items.Count - 1 do
    DstList.Items.AddObject(SrcList.Items[I],
      SrcList.Items.Objects[I]);
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
end;

procedure TBrigadeDlg.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do
    SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
  DstList.Items.Clear;
  SetItem(DstList, 0);
end;

procedure TBrigadeDlg.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
	begin
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
    end;
end;

procedure TBrigadeDlg.SetButtons;
var
  SrcEmpty, DstEmpty: Boolean;
begin
  SrcEmpty := SrcList.Items.Count = 0;
  DstEmpty := DstList.Items.Count = 0;
  IncludeBtn.Enabled := not SrcEmpty;
  IncAllBtn.Enabled := not SrcEmpty;
  ExcludeBtn.Enabled := not DstEmpty;
  ExAllBtn.Enabled := not DstEmpty;
end;

function TBrigadeDlg.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TBrigadeDlg.SetItem(List: TTextListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
	SetFocus;
	MaxIndex := List.Items.Count - 1;
	if Index = LB_ERR then Index := 0
	else if Index > MaxIndex then Index := MaxIndex;
	Selected[Index] := True;
  end;
  SetButtons;
end;

constructor TBrigadeDlg.Create(AOwner: TComponent; Table, BrigHosp : TTable;
	const BrgGID, RegGID, CityGID : Variant; WWDM: TWWDM);
var	Key : TFieldSet;
begin
	inherited Create(AOwner);

	HospGIDs := TStringList.Create;
	self.WWDM := WWDM;
	self.BrigHosp := BrigHosp;
	DBObj := TSimpleDBObject.Create(Table,'BrgGID');
	Key := TFieldSet.Create;
	try
		Key.Add('BrgGID',BrgGID);
		DBObj.Load(Key);
	finally
		Key.Free;
	end;
	DBObj.SetFieldValue('CityGID', CityGID);
	edName.Text := StringValue(DBObj.GetFieldValue('BrgName'));
	with WWDM.sqlAvailLPU do
	begin
		if RegGID=null then
			Filter := 'REGGID IS NULL'
		else Filter := 'REGGID='+IntToStr(RegGID);
		ParamByName('BRGGID').Value := BrgGID;
		Open;
		while not Eof do
		begin
			SrcList.Items.Add(FieldValues['MUFL']);
			HospGIDs.Add(FieldValues['MUFL']+'='+IntToStr(FieldValues['HspGID']));
			Next;
		end;
		Close;
	end;
	with WWDM.sqlLinkedLPU do
	begin
		ParamByName('BRGGID').Value := BrgGID;
		Open;
		while not Eof do
		begin
			DstList.Items.Add(FieldValues['MUFL']);
			HospGIDs.Add(FieldValues['MUFL']+'='+IntToStr(FieldValues['HspGID']));
			Next;
		end;
		Close;
	end;

	SetButtons;
end;

destructor TBrigadeDlg.Destroy;
begin
	HospGIDs.Free;
	DBObj.Free;
	inherited;
end;

procedure TBrigadeDlg.OKBtnClick(Sender: TObject);
var	BrgGID, i : Integer;
begin
	if edName.Text='' then
	begin
		ShowMessage(ERR_MSG_NO_NAME);
		ActiveControl := edName;
		Exit;
	end else DBObj.SetFieldValue('BrgName',edName.Text);
	DBObj.Save;
	BrgGID := DBObj.GetFieldValue('BRGGID');
	WWDM.sqlCleanBrigHosp.ParamByName('BRGGID').Value := BrgGID;
	WWDM.sqlCleanBrigHosp.ExecSQL;
	BrigHosp.Open;
	try
		BrigHosp.Edit;
		for i:=0 to DstList.Items.Count-1 do
		with BrigHosp do
			begin
				Append;
				FieldValues['BrgGID']:=BrgGID;
				FieldValues['HspGID']:=HospGIDs.Values[DstList.Items[i]];
				Post;
			end;
	finally
		BrigHosp.Close;
	end;
	ModalResult := mrOk;
end;

end.
